@@ -51,6 +51,7 @@ from alembic.testing import TestBase
 from alembic.testing.env import clear_staging_env
 from alembic.testing.env import staging_env
 from alembic.util import CommandError
+from ._autogen_fixtures import _default_name_filters
 from ._autogen_fixtures import _default_object_filters
 from ._autogen_fixtures import AutogenFixtureTest
 from ._autogen_fixtures import AutogenTest
@@ -98,6 +99,61 @@ class AutogenCrossSchemaTest(AutogenTest, TestBase):
         eq_(diffs[0][0], "add_table")
         eq_(diffs[0][1].schema, None)
 
+    def test_default_schema_omitted_by_table_name_upgrade(self):
+        def include_name(name, type_, parent_names):
+            if type_ == "table":
+                retval = name in ["t1", "t6"]
+                if retval:
+                    eq_(parent_names["schema_name"], None)
+                    eq_(parent_names["schema_qualified_table_name"], name)
+                else:
+                    eq_(parent_names["schema_name"], config.test_schema)
+                    eq_(
+                        parent_names["schema_qualified_table_name"],
+                        "%s.%s" % (config.test_schema, name),
+                    )
+                return retval
+            else:
+                return True
+
+        self._update_context(name_filters=include_name, include_schemas=True)
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(self.autogen_context, uo)
+
+        diffs = uo.as_diffs()
+        eq_(
+            {(d[0], d[1].name) for d in diffs},
+            {
+                ("add_table", "t3"),
+                ("add_table", "t4"),
+                ("remove_table", "t1"),
+                ("add_table", "t7"),
+            },
+        )
+
+    def test_default_schema_omitted_by_schema_name_upgrade(self):
+        def include_name(name, type_, parent_names):
+            if type_ == "schema":
+                assert not parent_names
+                return name is None
+            else:
+                return True
+
+        self._update_context(name_filters=include_name, include_schemas=True)
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(self.autogen_context, uo)
+
+        diffs = uo.as_diffs()
+        eq_(
+            {(d[0], d[1].name) for d in diffs},
+            {
+                ("add_table", "t3"),
+                ("add_table", "t4"),
+                ("remove_table", "t1"),
+                ("add_table", "t7"),
+            },
+        )
+
     def test_alt_schema_included_upgrade(self):
         def include_object(obj, name, type_, reflected, compare_to):
             if type_ == "table":
@@ -115,6 +171,32 @@ class AutogenCrossSchemaTest(AutogenTest, TestBase):
         eq_(diffs[0][0], "add_table")
         eq_(diffs[0][1].schema, config.test_schema)
 
+    def test_alt_schema_included_by_schema_name(self):
+        def include_name(name, type_, parent_names):
+            if type_ == "schema":
+                assert not parent_names
+                return name == config.test_schema
+            else:
+                return True
+
+        self._update_context(name_filters=include_name, include_schemas=True)
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(self.autogen_context, uo)
+
+        # does not include "t1" in drops because t1 is in default schema
+        # includes "t6" in adds because t6 is in default schema, was omitted,
+        # so reflection added it
+        diffs = uo.as_diffs()
+        eq_(
+            {(d[0], d[1].name) for d in diffs},
+            {
+                ("add_table", "t3"),
+                ("add_table", "t6"),
+                ("add_table", "t4"),
+                ("remove_table", "t2"),
+            },
+        )
+
     def test_default_schema_omitted_downgrade(self):
         def include_object(obj, name, type_, reflected, compare_to):
             if type_ == "table":
@@ -208,6 +290,10 @@ class AutogenDefaultSchemaIsNoneTest(AutogenFixtureTest, TestBase):
     def setUp(self):
         super(AutogenDefaultSchemaIsNoneTest, self).setUp()
 
+        # in SQLAlchemy 1.4, SQLite dialect is setting this name
+        # to "main" as is the actual default schema name for SQLite.
+        self.bind.dialect.default_schema_name = None
+
         # prerequisite
         eq_(self.bind.dialect.default_schema_name, None)
 
@@ -389,36 +475,6 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         eq_(diffs[10][3].table.name, "user")
         assert isinstance(diffs[10][3].type, String)
 
-    def test_include_symbol(self):
-
-        diffs = []
-
-        def include_symbol(name, schema=None):
-            return name in ("address", "order")
-
-        context = MigrationContext.configure(
-            connection=self.bind.connect(),
-            opts={
-                "compare_type": True,
-                "compare_server_default": True,
-                "target_metadata": self.m2,
-                "include_symbol": include_symbol,
-            },
-        )
-
-        diffs = autogenerate.compare_metadata(
-            context, context.opts["target_metadata"]
-        )
-
-        alter_cols = set(
-            [
-                d[2]
-                for d in self._flatten_diffs(diffs)
-                if d[0].startswith("modify")
-            ]
-        )
-        eq_(alter_cols, set(["order"]))
-
     def test_include_object(self):
         def include_object(obj, name, type_, reflected, compare_to):
             assert obj.name == name
@@ -472,6 +528,76 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         )
         eq_(alter_cols, set(["user_id", "order", "user"]))
 
+    def test_include_name(self):
+        all_names = set()
+
+        def include_name(name, type_, parent_names):
+            all_names.add((name, type_, parent_names.get("table_name", None)))
+            if type_ == "table":
+                eq_(
+                    parent_names,
+                    {"schema_name": None, "schema_qualified_table_name": name},
+                )
+                return name in ("address", "order", "user")
+            elif type_ == "column":
+                return name != "street"
+            else:
+                return True
+
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={
+                "compare_type": True,
+                "compare_server_default": True,
+                "target_metadata": self.m2,
+                "include_name": include_name,
+            },
+        )
+
+        diffs = autogenerate.compare_metadata(
+            context, context.opts["target_metadata"]
+        )
+        eq_(
+            all_names,
+            {
+                (None, "schema", None),
+                ("user", "table", None),
+                ("id", "column", "user"),
+                ("name", "column", "user"),
+                ("a1", "column", "user"),
+                ("pw", "column", "user"),
+                ("pw_idx", "index", "user"),
+                ("order", "table", None),
+                ("order_id", "column", "order"),
+                ("amount", "column", "order"),
+                ("address", "table", None),
+                ("id", "column", "address"),
+                ("email_address", "column", "address"),
+                ("extra", "table", None),
+            },
+        )
+
+        alter_cols = (
+            set(
+                [
+                    d[2]
+                    for d in self._flatten_diffs(diffs)
+                    if d[0].startswith("modify")
+                ]
+            )
+            .union(
+                d[3].name
+                for d in self._flatten_diffs(diffs)
+                if d[0] == "add_column"
+            )
+            .union(
+                d[1].name
+                for d in self._flatten_diffs(diffs)
+                if d[0] == "add_table"
+            )
+        )
+        eq_(alter_cols, {"user_id", "order", "user", "street", "item"})
+
     def test_skip_null_type_comparison_reflected(self):
         ac = ops.AlterColumnOp("sometable", "somecol")
         autogenerate.compare._compare_type(
@@ -750,6 +876,7 @@ class CompareMetadataToInspectorTest(TestBase):
             "alembic_module_prefix": "op.",
             "sqlalchemy_module_prefix": "sa.",
             "include_object": _default_object_filters,
+            "include_name": _default_name_filters,
         }
         if self.configure_opts:
             ctx_opts.update(self.configure_opts)
@@ -786,6 +913,8 @@ class CompareMetadataToInspectorTest(TestBase):
         # that have not changed.
         is_(self._compare_columns(cola, cola), False)
 
+    # TODO: ideally the backend-specific types would be tested
+    # within the test suites for those backends.
     @testing.combinations(
         (String(32), VARCHAR(32), False),
         (VARCHAR(6), String(6), False),
@@ -795,6 +924,40 @@ class CompareMetadataToInspectorTest(TestBase):
         (Unicode(32), VARCHAR(32), False, config.requirements.unicode_string),
         (VARCHAR(6), VARCHAR(12), True),
         (VARCHAR(6), String(12), True),
+        (Integer(), String(10), True),
+        (String(10), Integer(), True),
+        (
+            Unicode(30, collation="en_US"),
+            Unicode(30, collation="en_US"),
+            False,  # unfortunately dialects don't seem to consistently
+            # reflect collations right now so we can't test for
+            # positives here
+            config.requirements.postgresql,
+        ),
+        (
+            mysql.VARCHAR(200, charset="utf8"),
+            Unicode(200),
+            False,
+            config.requirements.mysql,
+        ),
+        (
+            mysql.VARCHAR(200, charset="latin1"),
+            mysql.VARCHAR(200, charset="utf-8"),
+            True,
+            config.requirements.mysql + config.requirements.sqlalchemy_13,
+        ),
+        (
+            String(255, collation="utf8_bin"),
+            String(255),
+            False,
+            config.requirements.mysql,
+        ),
+        (
+            String(255, collation="utf8_bin"),
+            String(255, collation="latin1_bin"),
+            True,
+            config.requirements.mysql + config.requirements.sqlalchemy_13,
+        ),
     )
     def test_string_comparisons(self, cola, colb, expect_changes):
         is_(self._compare_columns(cola, colb), expect_changes)
@@ -1285,42 +1448,62 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestBase):
         eq_(diffs[1][2], "order")
         eq_(diffs[1][3], metadata.tables["order"].c.user_id)
 
-    def test_compare_metadata_include_symbol(self):
+    def test_compare_metadata_include_name(self):
         metadata = self.m2
 
-        def include_symbol(table_name, schema_name):
-            return table_name in ("extra", "order")
+        all_names = set()
+
+        def include_name(name, type_, parent_names):
+            all_names.add((name, type_, parent_names.get("table_name", None)))
+            if type_ == "table":
+                return name in ("extra", "order")
+            elif type_ == "column":
+                return name != "amount"
+            else:
+                return True
 
         context = MigrationContext.configure(
             connection=self.bind.connect(),
             opts={
                 "compare_type": True,
                 "compare_server_default": True,
-                "include_symbol": include_symbol,
+                "include_name": include_name,
             },
         )
 
         diffs = autogenerate.compare_metadata(context, metadata)
+        eq_(
+            all_names,
+            {
+                ("user", "table", None),
+                ("order", "table", None),
+                ("address", "table", None),
+                (None, "schema", None),
+                ("amount", "column", "order"),
+                ("extra", "table", None),
+                ("order_id", "column", "order"),
+            },
+        )
 
-        eq_(diffs[0][0], "remove_table")
-        eq_(diffs[0][1].name, "extra")
-
-        eq_(diffs[1][0], "add_column")
-        eq_(diffs[1][1], None)
-        eq_(diffs[1][2], "order")
-        eq_(diffs[1][3], metadata.tables["order"].c.user_id)
-
-        eq_(diffs[2][0][0], "modify_type")
-        eq_(diffs[2][0][1], None)
-        eq_(diffs[2][0][2], "order")
-        eq_(diffs[2][0][3], "amount")
-        eq_(repr(diffs[2][0][5]), "NUMERIC(precision=8, scale=2)")
-        eq_(repr(diffs[2][0][6]), "Numeric(precision=10, scale=2)")
-
-        eq_(diffs[2][1][0], "modify_nullable")
-        eq_(diffs[2][1][2], "order")
-        eq_(diffs[2][1][5], False)
-        eq_(diffs[2][1][6], True)
+        eq_(
+            {
+                (
+                    d[0],
+                    d[3].name if d[0] == "add_column" else d[1].name,
+                    d[2] if d[0] == "add_column" else None,
+                )
+                for d in diffs
+            },
+            {
+                ("remove_table", "extra", None),
+                ("add_fk", None, None),
+                ("add_column", "amount", "order"),
+                ("add_table", "user", None),
+                ("add_table", "item", None),
+                ("add_column", "user_id", "order"),
+                ("add_table", "address", None),
+            },
+        )
 
     def test_compare_metadata_as_sql(self):
         context = MigrationContext.configure(
